@@ -1,4 +1,4 @@
-/*	$OpenBSD: cert.c,v 1.56 2022/02/04 16:50:49 tb Exp $ */
+/*	$OpenBSD: cert.c,v 1.62 2022/04/05 03:56:20 tb Exp $ */
 /*
  * Copyright (c) 2021 Job Snijders <job@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -305,6 +305,12 @@ sbgp_sia_resource(struct parse *p, const unsigned char *d, size_t dsz)
 			goto out;
 	}
 
+	if (p->res->mft == NULL || p->res->repo == NULL) {
+		warnx("%s: RFC 6487 section 4.8.8: SIA missing caRepository "
+		    "or rpkiManifest", p->fn);
+		goto out;
+	}
+
 	if (strstr(p->res->mft, p->res->repo) != p->res->mft) {
 		warnx("%s: RFC 6487 section 4.8.8: SIA: "
 		    "conflicting URIs for caRepository and rpkiManifest",
@@ -330,6 +336,12 @@ sbgp_sia(struct parse *p, X509_EXTENSION *ext)
 	const ASN1_TYPE		*t;
 	int			 dsz, rc = 0;
 
+	if (X509_EXTENSION_get_critical(ext)) {
+		warnx("%s: RFC 6487 section 4.8.8: SIA: "
+		    "extension not non-critical", p->fn);
+		goto out;
+	}
+
 	if ((dsz = i2d_X509_EXTENSION(ext, &sv)) < 0) {
 		cryptowarnx("%s: RFC 6487 section 4.8.8: SIA: "
 		    "failed extension parse", p->fn);
@@ -611,8 +623,6 @@ sbgp_assysnum(struct parse *p, X509_EXTENSION *ext)
 		goto out;
 	}
 
-	/* FIXME: verify OID. */
-
 	t = sk_ASN1_TYPE_value(seq, 1);
 	if (t->type != V_ASN1_BOOLEAN) {
 		warnx("%s: RFC 6487 section 4.8.11: autonomousSysNum: "
@@ -918,8 +928,6 @@ sbgp_ipaddrblk(struct parse *p, X509_EXTENSION *ext)
 		goto out;
 	}
 
-	/* FIXME: verify OID. */
-
 	t = sk_ASN1_TYPE_value(seq, 1);
 	if (t->type != V_ASN1_BOOLEAN) {
 		warnx("%s: RFC 6487 section 4.8.10: sbgp-ipAddrBlock: "
@@ -1052,13 +1060,10 @@ certificate_policies(struct parse *p, X509_EXTENSION *ext)
 /*
  * Parse and partially validate an RPKI X509 certificate (either a trust
  * anchor or a certificate) as defined in RFC 6487.
- * If "ta" is set, this is a trust anchor and must be self-signed.
- * Returns the parse results or NULL on failure ("xp" will be NULL too).
- * On success, free the pointer with cert_free() and make sure that "xp"
- * is also dereferenced.
+ * Returns the parse results or NULL on failure.
  */
-static struct cert *
-cert_parse_inner(const char *fn, const unsigned char *der, size_t len, int ta)
+struct cert *
+cert_parse_pre(const char *fn, const unsigned char *der, size_t len)
 {
 	int		 rc = 0, extsz, c;
 	int		 sia_present = 0;
@@ -1078,7 +1083,7 @@ cert_parse_inner(const char *fn, const unsigned char *der, size_t len, int ta)
 		err(1, NULL);
 
 	if ((x = d2i_X509(NULL, &der, len)) == NULL) {
-		cryptowarnx("%s: d2i_X509_bio", p.fn);
+		cryptowarnx("%s: d2i_X509", p.fn);
 		goto out;
 	}
 
@@ -1132,12 +1137,14 @@ cert_parse_inner(const char *fn, const unsigned char *der, size_t len, int ta)
 			goto out;
 	}
 
-	p.res->aki = x509_get_aki(x, ta, p.fn);
-	p.res->ski = x509_get_ski(x, p.fn);
-	if (!ta) {
-		p.res->aia = x509_get_aia(x, p.fn);
-		p.res->crl = x509_get_crl(x, p.fn);
-	}
+	if (!x509_get_aki(x, p.fn, &p.res->aki))
+		goto out;
+	if (!x509_get_ski(x, p.fn, &p.res->ski))
+		goto out;
+	if (!x509_get_aia(x, p.fn, &p.res->aia))
+		goto out;
+	if (!x509_get_crl(x, p.fn, &p.res->crl))
+		goto out;
 	if (!x509_get_expire(x, p.fn, &p.res->expires))
 		goto out;
 	p.res->purpose = x509_get_purpose(x, p.fn);
@@ -1171,11 +1178,6 @@ cert_parse_inner(const char *fn, const unsigned char *der, size_t len, int ta)
 			   p.fn);
 			goto out;
 		}
-		if (ta) {
-			warnx("%s: BGPsec cert can not be a trust anchor",
-			   p.fn);
-			goto out;
-		}
 		break;
 	default:
 		warnx("%s: x509_get_purpose failed in %s", p.fn, __func__);
@@ -1187,38 +1189,6 @@ cert_parse_inner(const char *fn, const unsigned char *der, size_t len, int ta)
 		goto out;
 	}
 
-	if (ta && p.res->aki != NULL && strcmp(p.res->aki, p.res->ski)) {
-		warnx("%s: RFC 6487 section 8.4.2: "
-		    "trust anchor AKI, if specified, must match SKI", p.fn);
-		goto out;
-	}
-
-	if (!ta && p.res->aki == NULL) {
-		warnx("%s: RFC 6487 section 8.4.2: "
-		    "non-trust anchor missing AKI", p.fn);
-		goto out;
-	} else if (!ta && strcmp(p.res->aki, p.res->ski) == 0) {
-		warnx("%s: RFC 6487 section 8.4.2: "
-		    "non-trust anchor AKI may not match SKI", p.fn);
-		goto out;
-	}
-
-	if (!ta && p.res->aia == NULL) {
-		warnx("%s: RFC 6487 section 8.4.7: "
-		    "non-trust anchor missing AIA", p.fn);
-		goto out;
-	} else if (ta && p.res->aia != NULL) {
-		warnx("%s: RFC 6487 section 8.4.7: "
-		    "trust anchor must not have AIA", p.fn);
-		goto out;
-	}
-
-	if (ta && p.res->crl != NULL) {
-		warnx("%s: RFC 6487 section 8.4.2: "
-		    "trust anchor may not specify CRL resource", p.fn);
-		goto out;
-	}
-
 	p.res->x509 = x;
 
 	rc = 1;
@@ -1231,22 +1201,40 @@ out:
 }
 
 struct cert *
-cert_parse(const char *fn, const unsigned char *der, size_t len)
+cert_parse(const char *fn, struct cert *p)
 {
-	return cert_parse_inner(fn, der, len, 0);
+	if (p->aki == NULL) {
+		warnx("%s: RFC 6487 section 8.4.2: "
+		    "non-trust anchor missing AKI", fn);
+		goto badcert;
+	}
+	if (strcmp(p->aki, p->ski) == 0) {
+		warnx("%s: RFC 6487 section 8.4.2: "
+		    "non-trust anchor AKI may not match SKI", fn);
+		goto badcert;
+	}
+	if (p->aia == NULL) {
+		warnx("%s: RFC 6487 section 8.4.7: AIA: extension missing", fn);
+		goto badcert;
+	}
+	if (p->crl == NULL) {
+		warnx("%s: RFC 6487 section 4.8.6: CRL: "
+		    "no CRL distribution point extension", fn);
+		goto badcert;
+	}
+	return p;
+
+badcert:
+	cert_free(p);
+	return NULL;
 }
 
 struct cert *
-ta_parse(const char *fn, const unsigned char *der, size_t len,
-    const unsigned char *pkey, size_t pkeysz)
+ta_parse(const char *fn, struct cert *p, const unsigned char *pkey,
+    size_t pkeysz)
 {
 	ASN1_TIME	*notBefore, *notAfter;
-	EVP_PKEY	*pk = NULL, *opk = NULL;
-	struct cert	*p;
-	int		 rc = 0;
-
-	if ((p = cert_parse_inner(fn, der, len, 1)) == NULL)
-		return NULL;
+	EVP_PKEY	*pk, *opk;
 
 	/* first check pubkey against the one from the TAL */
 	pk = d2i_PUBKEY(NULL, &pkey, pkeysz);
@@ -1257,7 +1245,8 @@ ta_parse(const char *fn, const unsigned char *der, size_t len,
 	if ((opk = X509_get0_pubkey(p->x509)) == NULL) {
 		cryptowarnx("%s: RFC 6487 (trust anchor): missing pubkey", fn);
 		goto badcert;
-	} else if (EVP_PKEY_cmp(pk, opk) != 1) {
+	}
+	if (EVP_PKEY_cmp(pk, opk) != 1) {
 		cryptowarnx("%s: RFC 6487 (trust anchor): "
 		    "pubkey does not match TAL pubkey", fn);
 		goto badcert;
@@ -1279,17 +1268,33 @@ ta_parse(const char *fn, const unsigned char *der, size_t len,
 		warnx("%s: certificate has expired", fn);
 		goto badcert;
 	}
+	if (p->aki != NULL && strcmp(p->aki, p->ski)) {
+		warnx("%s: RFC 6487 section 8.4.2: "
+		    "trust anchor AKI, if specified, must match SKI", fn);
+		goto badcert;
+	}
+	if (p->aia != NULL) {
+		warnx("%s: RFC 6487 section 8.4.7: "
+		    "trust anchor must not have AIA", fn);
+		goto badcert;
+	}
+	if (p->crl != NULL) {
+		warnx("%s: RFC 6487 section 8.4.2: "
+		    "trust anchor may not specify CRL resource", fn);
+		goto badcert;
+	}
+	if (p->purpose == CERT_PURPOSE_BGPSEC_ROUTER) {
+		warnx("%s: BGPsec cert cannot be a trust anchor", fn);
+		goto badcert;
+	}
 
-	rc = 1;
+	EVP_PKEY_free(pk);
+	return p;
 
 badcert:
 	EVP_PKEY_free(pk);
-	if (rc == 0) {
-		cert_free(p);
-		p = NULL;
-	}
-
-	return p;
+	cert_free(p);
+	return NULL;
 }
 
 /*
