@@ -1,4 +1,4 @@
-/*	$OpenBSD: http.c,v 1.52 2022/01/23 12:09:24 claudio Exp $  */
+/*	$OpenBSD: http.c,v 1.54 2022/03/11 09:57:54 claudio Exp $  */
 /*
  * Copyright (c) 2020 Nils Fisher <nils_fisher@hotmail.com>
  * Copyright (c) 2020 Claudio Jeker <claudio@openbsd.org>
@@ -1233,6 +1233,7 @@ http_parse_header(struct http_connection *conn, char *buf)
 	} else if (strncasecmp(cp, LAST_MODIFIED,
 	    sizeof(LAST_MODIFIED) - 1) == 0) {
 		cp += sizeof(LAST_MODIFIED) - 1;
+		free(conn->last_modified);
 		if ((conn->last_modified = strdup(cp)) == NULL)
 			err(1, NULL);
 	}
@@ -1812,6 +1813,9 @@ proc_http(char *bind_addr, int fd)
 		timeout = INFTIM;
 		now = getmonotime();
 		LIST_FOREACH(conn, &active, entry) {
+			if (i >= NPFDS)
+				errx(1, "too many connections");
+
 			if (conn->io_time == 0)
 				conn->io_time = now + HTTP_IO_TIMEOUT;
 
@@ -1831,10 +1835,11 @@ proc_http(char *bind_addr, int fd)
 			pfds[i].events = conn->events;
 			conn->pfd = &pfds[i];
 			i++;
-			if (i > NPFDS)
-				errx(1, "too many connections");
 		}
 		LIST_FOREACH(conn, &idle, entry) {
+			if (i >= NPFDS)
+				errx(1, "too many connections");
+
 			if (conn->idle_time <= now)
 				timeout = 0;
 			else {
@@ -1847,8 +1852,6 @@ proc_http(char *bind_addr, int fd)
 			pfds[i].events = POLLIN;
 			conn->pfd = &pfds[i];
 			i++;
-			if (i > NPFDS)
-				errx(1, "too many connections");
 		}
 
 		if (poll(pfds, i, timeout) == -1) {
