@@ -1,4 +1,4 @@
-/*	$OpenBSD: main.c,v 1.187 2022/01/28 15:30:23 claudio Exp $ */
+/*	$OpenBSD: main.c,v 1.192 2022/04/04 16:02:54 claudio Exp $ */
 /*
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -44,11 +44,6 @@
 #include "extern.h"
 #include "version.h"
 
-/*
- * Maximum number of TAL files we'll load.
- */
-#define	TALSZ_MAX	8
-
 const char	*tals[TALSZ_MAX];
 const char	*taldescs[TALSZ_MAX];
 unsigned int	 talrepocnt[TALSZ_MAX];
@@ -353,9 +348,6 @@ queue_add_from_mft(const char *path, const struct mftfile *file,
  * Loops over queue_add_from_mft() for all files.
  * The order here is important: we want to parse the revocation
  * list *before* we parse anything else.
- * FIXME: set the type of file in the mftfile so that we don't need to
- * keep doing the check (this should be done in the parser, where we
- * check the suffix anyway).
  */
 static void
 queue_add_from_mft_set(const struct mft *mft, const char *name, struct repo *rp)
@@ -695,7 +687,8 @@ check_fs_size(int fd, const char *cachedir)
 	if (fstatvfs(fd, &fs) == -1)
 		err(1, "statfs %s", cachedir);
 
-	if (fs.f_bavail < minsize / fs.f_frsize || fs.f_favail < minnode) {
+	if (fs.f_bavail < minsize / fs.f_frsize ||
+	    (fs.f_ffree > 0 && fs.f_favail < minnode)) {
 		fprintf(stderr, "WARNING: rpki-client may need more than "
 		    "the available disk space\n"
 		    "on the file-system holding %s.\n", cachedir);
@@ -1242,7 +1235,8 @@ main(int argc, char *argv[])
 
 	logx("all files parsed: generating output");
 
-	repo_cleanup(&fpt);
+	if (!noop)
+		repo_cleanup(&fpt, cachefd);
 
 	gettimeofday(&now_time, NULL);
 	timersub(&now_time, &start_time, &stats.elapsed_time);
