@@ -1,4 +1,4 @@
-/*	$OpenBSD: parser.c,v 1.60 2022/02/04 16:29:43 tb Exp $ */
+/*	$OpenBSD: parser.c,v 1.66 2022/04/02 12:17:53 claudio Exp $ */
 /*
  * Copyright (c) 2019 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -42,11 +42,14 @@
 static void		 build_chain(const struct auth *, STACK_OF(X509) **);
 static struct crl	*get_crl(const struct auth *);
 static void		 build_crls(const struct crl *, STACK_OF(X509_CRL) **);
+static struct crl	*parse_load_crl_from_mft(struct entity *, const char *);
 
 static X509_STORE_CTX	*ctx;
 static struct auth_tree	 auths = RB_INITIALIZER(&auths);
 static struct crl_tree	 crlt = RB_INITIALIZER(&crlt);
 
+struct tal		*talobj[TALSZ_MAX];
+
 extern ASN1_OBJECT	*certpol_oid;
 
 struct parse_repo {
@@ -93,19 +96,6 @@ repo_add(unsigned int id, char *path, char *validpath)
 		errx(1, "repository already added: id %d, %s", id, path);
 }
 
-static char *
-time2str(time_t t)
-{
-	static char buf[64];
-	struct tm tm;
-
-	if (gmtime_r(&t, &tm) == NULL)
-		return "could not convert time";
-
-	strftime(buf, sizeof(buf), "%h %d %T %Y %Z", &tm);
-	return buf;
-}
-
 /*
  * Build access path to file based on repoid, path, location and file values.
  */
@@ -205,13 +195,13 @@ verify_cb(int ok, X509_STORE_CTX *store_ctx)
  * Returns 1 for valid certificates, returns 0 if there is a verify error
  */
 static int
-valid_x509(char *file, X509 *x509, struct auth *a, struct crl *crl,
-    unsigned long flags, int nowarn)
+valid_x509(char *file, X509 *x509, struct auth *a, struct crl *crl, int nowarn)
 {
 	X509_VERIFY_PARAM	*params;
 	ASN1_OBJECT		*cp_oid;
 	STACK_OF(X509)		*chain;
 	STACK_OF(X509_CRL)	*crls = NULL;
+	unsigned long		 flags;
 	int			 c;
 
 	build_chain(a, &chain);
@@ -231,6 +221,7 @@ valid_x509(char *file, X509 *x509, struct auth *a, struct crl *crl,
 	X509_STORE_CTX_set_verify_cb(ctx, verify_cb);
 	if (!X509_STORE_CTX_set_app_data(ctx, file))
 		cryptoerrx("X509_STORE_CTX_set_app_data");
+	flags = X509_V_FLAG_CRL_CHECK;
 	flags |= X509_V_FLAG_EXPLICIT_POLICY;
 	flags |= X509_V_FLAG_INHIBIT_MAP;
 	X509_STORE_CTX_set_flags(ctx, flags);
@@ -263,8 +254,8 @@ static struct roa *
 proc_parser_roa(char *file, const unsigned char *der, size_t len)
 {
 	struct roa		*roa;
-	struct crl		*crl;
 	struct auth		*a;
+	struct crl		*crl;
 	X509			*x509;
 
 	if ((roa = roa_parse(&x509, file, der, len)) == NULL)
@@ -273,7 +264,7 @@ proc_parser_roa(char *file, const unsigned char *der, size_t len)
 	a = valid_ski_aki(file, &auths, roa->ski, roa->aki);
 	crl = get_crl(a);
 
-	if (!valid_x509(file, x509, a, crl, X509_V_FLAG_CRL_CHECK, 0)) {
+	if (!valid_x509(file, x509, a, crl, 0)) {
 		X509_free(x509);
 		roa_free(roa);
 		return NULL;
@@ -354,32 +345,46 @@ proc_parser_mft_check(const char *fn, struct mft *p)
  * Parse and validate a manifest file. Skip checking the fileandhash
  * this is done in the post check. After this step we know the mft is
  * valid and can be compared.
- * Here we *don't* validate against the list of CRLs, because the
- * certificate used to sign the manifest may specify a CRL that the root
- * certificate didn't, and we haven't scanned for it yet.
- * This chicken-and-egg isn't important, however, because we'll catch
- * the revocation list by the time we scan for any contained resources
- * (ROA, CER) and will see it then.
  * Return the mft on success or NULL on failure.
  */
 static struct mft *
-proc_parser_mft_pre(char *file, const unsigned char *der, size_t len)
+proc_parser_mft_pre(char *file, const unsigned char *der, size_t len,
+    struct entity *entp)
 {
-	struct mft		*mft;
-	X509			*x509;
-	struct auth		*a;
+	struct mft	*mft;
+	X509		*x509;
+	struct crl	*crl;
+	struct auth	*a;
+	char		*c, *crlfile;
 
 	if ((mft = mft_parse(&x509, file, der, len)) == NULL)
 		return NULL;
 
 	a = valid_ski_aki(file, &auths, mft->ski, mft->aki);
-
-	/* CRL checks disabled here because CRL is referenced from mft */
-	if (!valid_x509(file, x509, a, NULL, 0, 1)) {
+	/* load CRL by hand, since it is referenced by the MFT itself */
+	if (!x509_get_crl(x509, file, &c) || c == NULL) {
+		if (c == NULL)
+			warnx("%s: RFC 6487 section 4.8.6: CRL: "
+			    "no CRL distribution point extension", file);
 		mft_free(mft);
 		X509_free(x509);
 		return NULL;
 	}
+	crlfile = strrchr(c, '/');
+	if (crlfile != NULL)
+		crlfile++;
+	else
+		crlfile = c;
+	crl = parse_load_crl_from_mft(entp, crlfile);
+	free(c);
+
+	if (!valid_x509(file, x509, a, crl, 1)) {
+		mft_free(mft);
+		crl_free(crl);
+		X509_free(x509);
+		return NULL;
+	}
+	crl_free(crl);
 	X509_free(x509);
 
 	return mft;
@@ -440,7 +445,7 @@ proc_parser_cert_validate(char *file, struct cert *cert)
 	a = valid_ski_aki(file, &auths, cert->ski, cert->aki);
 	crl = get_crl(a);
 
-	if (!valid_x509(file, cert->x509, a, crl, X509_V_FLAG_CRL_CHECK, 0)) {
+	if (!valid_x509(file, cert->x509, a, crl, 0)) {
 		cert_free(cert);
 		return NULL;
 	}
@@ -476,7 +481,10 @@ proc_parser_cert(char *file, const unsigned char *der, size_t len)
 
 	/* Extract certificate data. */
 
-	cert = cert_parse(file, der, len);
+	cert = cert_parse_pre(file, der, len);
+	if (cert == NULL)
+		return NULL;
+	cert = cert_parse(file, cert);
 	if (cert == NULL)
 		return NULL;
 
@@ -501,7 +509,10 @@ proc_parser_root_cert(char *file, const unsigned char *der, size_t len,
 
 	/* Extract certificate data. */
 
-	cert = ta_parse(file, der, len, pkey, pkeysz);
+	cert = cert_parse_pre(file, der, len);
+	if (cert == NULL)
+		return NULL;
+	cert = ta_parse(file, cert, pkey, pkeysz);
 	if (cert == NULL)
 		return NULL;
 
@@ -530,45 +541,16 @@ proc_parser_root_cert(char *file, const unsigned char *der, size_t len,
 static void
 proc_parser_crl(char *file, const unsigned char *der, size_t len)
 {
-	X509_CRL		*x509_crl;
-	struct crl		*crl;
-	const ASN1_TIME		*at;
-	struct tm		 expires_tm;
+	struct crl	*crl;
 
-	if ((x509_crl = crl_parse(file, der, len)) != NULL) {
-		if ((crl = malloc(sizeof(*crl))) == NULL)
-			err(1, NULL);
-		if ((crl->aki = x509_crl_get_aki(x509_crl, file)) == NULL) {
-			warnx("x509_crl_get_aki failed");
-			goto err;
-		}
+	if ((crl = crl_parse(file, der, len)) == NULL)
+		return;
 
-		crl->x509_crl = x509_crl;
-
-		/* extract expire time for later use */
-		at = X509_CRL_get0_nextUpdate(x509_crl);
-		if (at == NULL) {
-			warnx("%s: X509_CRL_get0_nextUpdate failed", file);
-			goto err;
-		}
-		memset(&expires_tm, 0, sizeof(expires_tm));
-		if (ASN1_time_parse(at->data, at->length, &expires_tm,
-		    0) == -1) {
-			warnx("%s: ASN1_time_parse failed", file);
-			goto err;
-		}
-		if ((crl->expires = mktime(&expires_tm)) == -1)
-			errx(1, "%s: mktime failed", file);
-
-		if (RB_INSERT(crl_tree, &crlt, crl) != NULL) {
-			if (!filemode)
-				warnx("%s: duplicate AKI %s", file, crl->aki);
-			goto err;
-		}
+	if (RB_INSERT(crl_tree, &crlt, crl) != NULL) {
+		if (!filemode)
+			warnx("%s: duplicate AKI %s", file, crl->aki);
+		crl_free(crl);
 	}
-	return;
- err:
-	free_crl(crl);
 }
 
 /*
@@ -579,8 +561,8 @@ proc_parser_gbr(char *file, const unsigned char *der, size_t len)
 {
 	struct gbr		*gbr;
 	X509			*x509;
-	struct auth		*a;
 	struct crl		*crl;
+	struct auth		*a;
 
 	if ((gbr = gbr_parse(&x509, file, der, len)) == NULL)
 		return;
@@ -589,7 +571,7 @@ proc_parser_gbr(char *file, const unsigned char *der, size_t len)
 	crl = get_crl(a);
 
 	/* return value can be ignored since nothing happens here */
-	valid_x509(file, x509, a, crl, X509_V_FLAG_CRL_CHECK, 0);
+	valid_x509(file, x509, a, crl, 0);
 
 	X509_free(x509);
 	gbr_free(gbr);
@@ -627,7 +609,6 @@ get_crl(const struct auth *a)
 
 	if (a == NULL)
 		return NULL;
-
 	find.aki = a->cert->ski;
 	return RB_FIND(crl_tree, &crlt, &find);
 }
@@ -643,10 +624,8 @@ build_crls(const struct crl *crl, STACK_OF(X509_CRL) **crls)
 
 	if (crl == NULL)
 		return;
-
 	if ((*crls = sk_X509_CRL_new_null()) == NULL)
 		errx(1, "sk_X509_CRL_new_null");
-
 	if (!sk_X509_CRL_push(*crls, crl->x509_crl))
 		err(1, "sk_X509_CRL_push");
 }
@@ -671,6 +650,9 @@ parse_load_file(struct entity *entp, unsigned char **f, size_t *flen)
 	return file;
 }
 
+/*
+ * Load the most recent MFT by opening both options and comparing the two.
+ */
 static char *
 parse_load_mft(struct entity *entp, struct mft **mft)
 {
@@ -685,7 +667,7 @@ parse_load_mft(struct entity *entp, struct mft **mft)
 		f = load_file(file1, &flen);
 		if (f == NULL && errno != ENOENT)
 			warn("parse file %s", file1);
-		mft1 = proc_parser_mft_pre(file1, f, flen);
+		mft1 = proc_parser_mft_pre(file1, f, flen, entp);
 		free(f);
 	}
 
@@ -693,7 +675,7 @@ parse_load_mft(struct entity *entp, struct mft **mft)
 		f = load_file(file2, &flen);
 		if (f == NULL && errno != ENOENT)
 			warn("parse file %s", file2);
-		mft2 = proc_parser_mft_pre(file2, f, flen);
+		mft2 = proc_parser_mft_pre(file2, f, flen, entp);
 		free(f);
 	}
 
@@ -710,6 +692,54 @@ parse_load_mft(struct entity *entp, struct mft **mft)
 	}
 }
 
+/*
+ * Load the most recent CRL by opening both options and comparing the two.
+ */
+static struct crl *
+parse_load_crl_from_mft(struct entity *entp, const char *file)
+{
+	struct crl	*crl1 = NULL, *crl2 = NULL;
+	char		*file1, *file2;
+	unsigned char	*f;
+	size_t		 flen;
+
+	if (file == NULL)
+		return NULL;
+
+	file1 = parse_filepath(entp->repoid, entp->path, file, DIR_VALID);
+	file2 = parse_filepath(entp->repoid, entp->path, file, DIR_TEMP);
+
+	if (file1 != NULL) {
+		f = load_file(file1, &flen);
+		if (f == NULL && errno != ENOENT)
+			warn("parse file %s", file1);
+		crl1 = crl_parse(file1, f, flen);
+		free(f);
+	}
+
+	if (file2 != NULL) {
+		f = load_file(file2, &flen);
+		if (f == NULL && errno != ENOENT)
+			warn("parse file %s", file2);
+		crl2 = crl_parse(file2, f, flen);
+		free(f);
+	}
+
+	free(file1);
+	free(file2);
+
+	if (crl1 == NULL || crl2 == NULL)
+		return crl2 == NULL ? crl1 : crl2;
+
+	if (crl1->issued >= crl2->issued) {
+		crl_free(crl2);
+		return crl1;
+	} else {
+		crl_free(crl1);
+		return crl2;
+	}
+}
+
 /*
  * Process an entity and responing to parent process.
  */
@@ -874,7 +904,7 @@ parse_load_cert(char *uri)
 		goto done;
 	}
 
-	cert = cert_parse(uri, f, flen);
+	cert = cert_parse_pre(uri, f, flen);
 	free(f);
 
 	if (cert == NULL)
@@ -972,6 +1002,35 @@ parse_load_ta(struct tal *tal)
 	free(f);
 }
 
+static struct tal *
+find_tal(struct cert *cert)
+{
+	EVP_PKEY	*pk, *opk;
+	struct tal	*tal;
+	int		 i;
+
+	if ((opk = X509_get0_pubkey(cert->x509)) == NULL)
+		return NULL;
+
+	for (i = 0; i < TALSZ_MAX; i++) {
+		const unsigned char *pkey;
+
+		if (talobj[i] == NULL)
+			break;
+		tal = talobj[i];
+		pkey = tal->pkey;
+		pk = d2i_PUBKEY(NULL, &pkey, tal->pkeysz);
+		if (pk == NULL)
+			continue;
+		if (EVP_PKEY_cmp(pk, opk) == 1) {
+			EVP_PKEY_free(pk);
+			return tal;
+		}
+		EVP_PKEY_free(pk);
+	}
+	return NULL;
+}
+
 /*
  * Parse file passed with -f option.
  */
@@ -981,13 +1040,14 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 	static int num;
 	X509 *x509 = NULL;
 	struct cert *cert = NULL;
+	struct crl *crl = NULL;
 	struct mft *mft = NULL;
 	struct roa *roa = NULL;
 	struct gbr *gbr = NULL;
 	struct tal *tal = NULL;
-	enum rtype type;
 	char *aia = NULL, *aki = NULL;
-	unsigned long verify_flags = X509_V_FLAG_CRL_CHECK;
+	enum rtype type;
+	int is_ta = 0;
 
 	if (num++ > 0)
 		printf("--\n");
@@ -1007,7 +1067,12 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 
 	switch (type) {
 	case RTYPE_CER:
-		cert = cert_parse(file, buf, len);
+		cert = cert_parse_pre(file, buf, len);
+		if (cert == NULL)
+			break;
+		is_ta = X509_get_extension_flags(cert->x509) & EXFLAG_SS;
+		if (!is_ta)
+			cert = cert_parse(file, cert);
 		if (cert == NULL)
 			break;
 		cert_print(cert);
@@ -1017,6 +1082,12 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 		if (X509_up_ref(x509) == 0)
 			errx(1, "%s: X509_up_ref failed", __func__);
 		break;
+	case RTYPE_CRL:
+		crl = crl_parse(file, buf, len);
+		if (crl == NULL)
+			break;
+		crl_print(crl);
+		break;
 	case RTYPE_MFT:
 		mft = mft_parse(&x509, file, buf, len);
 		if (mft == NULL)
@@ -1024,7 +1095,6 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 		mft_print(mft);
 		aia = mft->aia;
 		aki = mft->aki;
-		verify_flags = 0;
 		break;
 	case RTYPE_ROA:
 		roa = roa_parse(&x509, file, buf, len);
@@ -1048,7 +1118,6 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 			break;
 		tal_print(tal);
 		break;
-	case RTYPE_CRL: /* XXX no printer yet */
 	default:
 		printf("%s: unsupported file type\n", file);
 		break;
@@ -1056,17 +1125,32 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 
 	if (aia != NULL) {
 		struct auth *a;
-		struct crl *crl;
-		char *c;
+		struct crl *c;
+		char *crl_uri;
 
-		c = x509_get_crl(x509, file);
-		parse_load_crl(c);
-		free(c);
-		parse_load_certchain(aia);
+		x509_get_crl(x509, file, &crl_uri);
+		parse_load_crl(crl_uri);
+		free(crl_uri);
+		if (auth_find(&auths, aki) == NULL)
+			parse_load_certchain(aia);
 		a = auth_find(&auths, aki);
-		crl = get_crl(a);
+		c = get_crl(a);
 
-		if (valid_x509(file, x509, a, crl, verify_flags, 0))
+		if (valid_x509(file, x509, a, c, 0))
+			printf("Validation: OK\n");
+		else
+			printf("Validation: Failed\n");
+	} else if (is_ta) {
+		if ((tal = find_tal(cert)) != NULL) {
+			cert = ta_parse(file, cert, tal->pkey, tal->pkeysz);
+			printf("TAL: %s\n", tal->descr);
+			tal = NULL;
+		} else {
+			cert_free(cert);
+			cert = NULL;
+			printf("TAL: not found\n");
+		}
+		if (cert != NULL)
 			printf("Validation: OK\n");
 		else
 			printf("Validation: Failed\n");
@@ -1074,6 +1158,7 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 
 	X509_free(x509);
 	cert_free(cert);
+	crl_free(crl);
 	mft_free(mft);
 	roa_free(roa);
 	gbr_free(gbr);
@@ -1103,8 +1188,8 @@ parse_file(struct entityq *q, struct msgbuf *msgq)
 				errx(1, "%s: could not parse tal file",
 				    entp->file);
 			tal->id = entp->talid;
+			talobj[tal->id] = tal;
 			parse_load_ta(tal);
-			tal_free(tal);
 			break;
 		default:
 			errx(1, "unhandled entity type %d", entp->type);
