@@ -1,4 +1,4 @@
-/*	$OpenBSD: rrdp.c,v 1.21 2022/01/23 12:09:24 claudio Exp $ */
+/*	$OpenBSD: rrdp.c,v 1.23 2022/04/04 13:45:02 claudio Exp $ */
 /*
  * Copyright (c) 2020 Nils Fisher <nils_fisher@hotmail.com>
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
@@ -256,7 +256,6 @@ rrdp_failed(struct rrdp *s)
 	/* reset file state before retrying */
 	s->file_failed = 0;
 
-	/* XXX MUST do some cleanup in the repo here */
 	if (s->task == DELTA) {
 		/* fallback to a snapshot as per RFC8182 */
 		free_delta_xml(s->dxml);
@@ -429,6 +428,7 @@ rrdp_input_handler(int fd)
 			errx(1, "%s: bad internal state", s->local);
 
 		s->res = res;
+		free(s->last_mod);
 		s->last_mod = last_mod;
 		s->state |= RRDP_STATE_HTTP_DONE;
 		rrdp_finished(s);
