@@ -1,4 +1,4 @@
-/*	$OpenBSD: x509.c,v 1.34 2022/02/04 16:08:53 tb Exp $ */
+/*	$OpenBSD: x509.c,v 1.38 2022/04/01 17:22:07 claudio Exp $ */
 /*
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -38,6 +38,10 @@ ASN1_OBJECT	*roa_oid;	/* id-ct-routeOriginAuthz CMS content type */
 ASN1_OBJECT	*mft_oid;	/* id-ct-rpkiManifest CMS content type */
 ASN1_OBJECT	*gbr_oid;	/* id-ct-rpkiGhostbusters CMS content type */
 ASN1_OBJECT	*bgpsec_oid;	/* id-kp-bgpsec-router Key Purpose */
+ASN1_OBJECT	*cnt_type_oid;	/* pkcs-9 id-contentType */
+ASN1_OBJECT	*msg_dgst_oid;	/* pkcs-9 id-messageDigest */
+ASN1_OBJECT	*sign_time_oid;	/* pkcs-9 id-signingTime */
+ASN1_OBJECT	*bin_sign_time_oid;	/* pkcs-9 id-aa-binarySigningTime */
 
 void
 x509_init_oid(void)
@@ -62,6 +66,16 @@ x509_init_oid(void)
 		    "1.2.840.113549.1.9.16.1.35");
 	if ((bgpsec_oid = OBJ_txt2obj("1.3.6.1.5.5.7.3.30", 1)) == NULL)
 		errx(1, "OBJ_txt2obj for %s failed", "1.3.6.1.5.5.7.3.30");
+	if ((cnt_type_oid = OBJ_txt2obj("1.2.840.113549.1.9.3", 1)) == NULL)
+		errx(1, "OBJ_txt2obj for %s failed", "1.2.840.113549.1.9.3");
+	if ((msg_dgst_oid = OBJ_txt2obj("1.2.840.113549.1.9.4", 1)) == NULL)
+		errx(1, "OBJ_txt2obj for %s failed", "1.2.840.113549.1.9.4");
+	if ((sign_time_oid = OBJ_txt2obj("1.2.840.113549.1.9.5", 1)) == NULL)
+		errx(1, "OBJ_txt2obj for %s failed", "1.2.840.113549.1.9.5");
+	if ((bin_sign_time_oid =
+	    OBJ_txt2obj("1.2.840.113549.1.9.16.2.46", 1)) == NULL)
+		errx(1, "OBJ_txt2obj for %s failed",
+		    "1.2.840.113549.1.9.16.2.46");
 }
 
 /*
@@ -69,22 +83,18 @@ x509_init_oid(void)
  * Returns the AKI or NULL if it could not be parsed.
  * The AKI is formatted as a hex string.
  */
-char *
-x509_get_aki(X509 *x, int ta, const char *fn)
+int
+x509_get_aki(X509 *x, const char *fn, char **aki)
 {
 	const unsigned char	*d;
 	AUTHORITY_KEYID		*akid;
 	ASN1_OCTET_STRING	*os;
-	int			 dsz, crit;
-	char			*res = NULL;
+	int			 dsz, crit, rc = 0;
 
+	*aki = NULL;
 	akid = X509_get_ext_d2i(x, NID_authority_key_identifier, &crit, NULL);
-	if (akid == NULL) {
-		if (!ta)
-			warnx("%s: RFC 6487 section 4.8.3: AKI: "
-			    "extension missing", fn);
-		return NULL;
-	}
+	if (akid == NULL)
+		return 1;
 	if (crit != 0) {
 		warnx("%s: RFC 6487 section 4.8.3: "
 		    "AKI: extension not non-critical", fn);
@@ -114,11 +124,11 @@ x509_get_aki(X509 *x, int ta, const char *fn)
 		goto out;
 	}
 
-	res = hex_encode(d, dsz);
-
+	*aki = hex_encode(d, dsz);
+	rc = 1;
 out:
 	AUTHORITY_KEYID_free(akid);
-	return res;
+	return rc;
 }
 
 /*
@@ -126,19 +136,17 @@ out:
  * Returns the SKI or NULL if it could not be parsed.
  * The SKI is formatted as a hex string.
  */
-char *
-x509_get_ski(X509 *x, const char *fn)
+int
+x509_get_ski(X509 *x, const char *fn, char **ski)
 {
 	const unsigned char	*d;
 	ASN1_OCTET_STRING	*os;
-	int			 dsz, crit;
-	char			*res = NULL;
+	int			 dsz, crit, rc = 0;
 
+	*ski = NULL;
 	os = X509_get_ext_d2i(x, NID_subject_key_identifier, &crit, NULL);
-	if (os == NULL) {
-		warnx("%s: RFC 6487 section 4.8.2: SKI: extension missing", fn);
-		return NULL;
-	}
+	if (os == NULL)
+		return 1;
 	if (crit != 0) {
 		warnx("%s: RFC 6487 section 4.8.2: "
 		    "SKI: extension not non-critical", fn);
@@ -155,10 +163,11 @@ x509_get_ski(X509 *x, const char *fn)
 		goto out;
 	}
 
-	res = hex_encode(d, dsz);
+	*ski = hex_encode(d, dsz);
+	rc = 1;
 out:
 	ASN1_OCTET_STRING_free(os);
-	return res;
+	return rc;
 }
 
 /*
@@ -267,19 +276,18 @@ x509_get_pubkey(X509 *x, const char *fn)
  * Returns NULL on failure, on success returns the AIA URI
  * (which has to be freed after use).
  */
-char *
-x509_get_aia(X509 *x, const char *fn)
+int
+x509_get_aia(X509 *x, const char *fn, char **aia)
 {
 	ACCESS_DESCRIPTION		*ad;
 	AUTHORITY_INFO_ACCESS		*info;
-	char				*aia = NULL;
-	int				 crit;
+	int				 crit, rc = 0;
 
+	*aia = NULL;
 	info = X509_get_ext_d2i(x, NID_info_access, &crit, NULL);
-	if (info == NULL) {
-		warnx("%s: RFC 6487 section 4.8.7: AIA: extension missing", fn);
-		return NULL;
-	}
+	if (info == NULL)
+		return 1;
+
 	if (crit != 0) {
 		warnx("%s: RFC 6487 section 4.8.7: "
 		    "AIA: extension not non-critical", fn);
@@ -311,15 +319,16 @@ x509_get_aia(X509 *x, const char *fn)
 		goto out;
 	}
 
-	aia = strndup(
+	*aia = strndup(
 	    ASN1_STRING_get0_data(ad->location->d.uniformResourceIdentifier),
 	    ASN1_STRING_length(ad->location->d.uniformResourceIdentifier));
-	if (aia == NULL)
+	if (*aia == NULL)
 		err(1, NULL);
+	rc = 1;
 
 out:
 	AUTHORITY_INFO_ACCESS_free(info);
-	return aia;
+	return rc;
 }
 
 /*
@@ -329,23 +338,16 @@ int
 x509_get_expire(X509 *x, const char *fn, time_t *tt)
 {
 	const ASN1_TIME	*at;
-	struct tm	 expires_tm;
-	time_t		 expires;
 
 	at = X509_get0_notAfter(x);
 	if (at == NULL) {
 		warnx("%s: X509_get0_notafter failed", fn);
 		return 0;
 	}
-	memset(&expires_tm, 0, sizeof(expires_tm));
-	if (ASN1_time_parse(at->data, at->length, &expires_tm, 0) == -1) {
+	if (x509_get_time(at, tt) == -1) {
 		warnx("%s: ASN1_time_parse failed", fn);
 		return 0;
 	}
-	if ((expires = mktime(&expires_tm)) == -1)
-		errx(1, "%s: mktime failed", fn);
-
-	*tt = expires;
 	return 1;
 
 }
@@ -357,21 +359,19 @@ x509_get_expire(X509 *x, const char *fn, time_t *tt)
  * Returns NULL on failure, the crl URI on success which has to be freed
  * after use.
  */
-char *
-x509_get_crl(X509 *x, const char *fn)
+int
+x509_get_crl(X509 *x, const char *fn, char **crl)
 {
 	CRL_DIST_POINTS		*crldp;
 	DIST_POINT		*dp;
 	GENERAL_NAME		*name;
-	char			*crl = NULL;
-	int			 crit;
+	int			 crit, rc = 0;
 
+	*crl = NULL;
 	crldp = X509_get_ext_d2i(x, NID_crl_distribution_points, &crit, NULL);
-	if (crldp == NULL) {
-		warnx("%s: RFC 6487 section 4.8.6: CRL: "
-		    "no CRL distribution point extension", fn);
-		return NULL;
-	}
+	if (crldp == NULL)
+		return 1;
+
 	if (crit != 0) {
 		warnx("%s: RFC 6487 section 4.8.6: "
 		    "CRL distribution point: extension not non-critical", fn);
@@ -418,14 +418,15 @@ x509_get_crl(X509 *x, const char *fn)
 		goto out;
 	}
 
-	crl = strndup(ASN1_STRING_get0_data(name->d.uniformResourceIdentifier),
+	*crl = strndup(ASN1_STRING_get0_data(name->d.uniformResourceIdentifier),
 	    ASN1_STRING_length(name->d.uniformResourceIdentifier));
-	if (crl == NULL)
+	if (*crl == NULL)
 		err(1, NULL);
+	rc = 1;
 
 out:
 	CRL_DIST_POINTS_free(crldp);
-	return crl;
+	return rc;
 }
 
 /*
@@ -483,3 +484,61 @@ out:
 	AUTHORITY_KEYID_free(akid);
 	return res;
 }
+
+/*
+ * Convert passed ASN1_TIME to time_t *t.
+ * Returns 1 on success and 0 on failure.
+ */
+int
+x509_get_time(const ASN1_TIME *at, time_t *t)
+{
+	struct tm	 tm;
+
+	*t = 0;
+	memset(&tm, 0, sizeof(tm));
+	if (ASN1_time_parse(at->data, at->length, &tm, 0) == -1)
+		return 0;
+	if ((*t = mktime(&tm)) == -1)
+		errx(1, "mktime failed");
+	return 1;
+}
+
+/*
+ * Convert an ASN1_INTEGER into a hexstring.
+ * Returned string needs to be freed by the caller.
+ */
+char *
+x509_convert_seqnum(const char *fn, const ASN1_INTEGER *i)
+{
+	BIGNUM	*seqnum = NULL;
+	char	*s = NULL;
+
+	if (i == NULL)
+		goto out;
+
+	seqnum = ASN1_INTEGER_to_BN(i, NULL);
+	if (seqnum == NULL) {
+		warnx("%s: ASN1_INTEGER_to_BN error", fn);
+		goto out;
+	}
+
+	if (BN_is_negative(seqnum)) {
+		warnx("%s: %s: want positive integer, have negative.",
+		    __func__, fn);
+		goto out;
+	}
+
+	if (BN_num_bytes(seqnum) > 20) {
+		warnx("%s: %s: want 20 octets or fewer, have more.",
+		    __func__, fn);
+		goto out;
+	}
+
+	s = BN_bn2hex(seqnum);
+	if (s == NULL)
+		warnx("%s: BN_bn2hex error", fn);
+
+ out:
+	BN_free(seqnum);
+	return s;
+}
